/*
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 *
 */
package com.sun.grizzly.http;

import com.sun.grizzly.arp.AsyncHandler;
import com.sun.grizzly.arp.DefaultAsyncHandler;
import com.sun.grizzly.comet.CometAsyncFilter;
import com.sun.grizzly.http.servlet.ServletAdapter;
import com.sun.grizzly.samples.comet.TwitterServlet;

import org.apache.jasper.servlet.JspServlet;

import javax.servlet.Servlet;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.logging.Logger;

/**
 * Test case for JSP issue.
 * Based on <a href="https://grizzly.dev.java.net/issues/show_bug.cgi?id=615">#615</a>.
 *
 * @author Hubert Iwaniuk
 */
public class JasperTest extends GrizzlyWebServerAbstractTest {

    //public static final int PORT = 18890 + 11;
	public static final int PORT = 8080;
    private static Logger logger = Logger.getLogger("grizzly.test");

    public void testJSP_Jasper() throws IOException {
        logger.info("testJSP_Jasper");

        try {
            startGrizzlyWebServer(PORT);
            String[] aliases = new String[]{"*.jsp"};

            String context = "/ctx";
            String servletPath = "/jsps";
            String rootFolder = ".";

            ServletAdapter adapter = new ServletAdapter();
            Servlet servlet = new JspServlet();
            adapter.setServletInstance(servlet);

            adapter.setContextPath(context);
            adapter.setServletPath(servletPath);
            adapter.setRootFolder(rootFolder);

            gws.addGrizzlyAdapter(adapter, aliases);

            gws.start();

            String url = context + servletPath + "/index.jsp";
            HttpURLConnection conn = getConnection(url, PORT);
            assertEquals(HttpServletResponse.SC_OK, getResponseCodeFromAlias(conn));

            String response = readResponse(conn);
            assertEquals(url, response.trim());

        } finally {
            stopGrizzlyWebServer();
        }
    }

	public void testJSP_TwitterOK() throws IOException {
	    logger.info("testJSP_TwitterOK");
	
	    try {
	        startGrizzlyWebServer(PORT);
	        
	        // TWITTER 
	        ServletAdapter adapter2 = new ServletAdapter();
	        Servlet servlet2 = new TwitterServlet();
	        adapter2.setServletInstance(servlet2);
	        adapter2.addInitParameter("cometUrlPattern", "/griztter");
	        adapter2.addInitParameter("expirationDelay", "600000");
	
	        adapter2.setContextPath("/grizzly-twitter");
	        adapter2.setServletPath("/griztter");
	        adapter2.setRootFolder("c:/temp/demo/grizzly-twitter");
	
	        gws.addGrizzlyAdapter(adapter2, new String[]{"/grizzly-twitter/griztter"});
	        
	        
	        ServletAdapter adapter3 = new ServletAdapter();
    		
	        adapter3.setContextPath("/grizzly-twitter");
    		adapter3.setServletPath("/");
    		adapter3.setHandleStaticResources(true);
    		adapter3.setRootFolder("c:/temp/demo/grizzly-twitter");
    		
    		gws.addGrizzlyAdapter(adapter3, new String[]{"/grizzly-twitter/"});
	        
	        
	        SelectorThread st = gws.getSelectorThread();
            
            AsyncHandler asyncHandler = new DefaultAsyncHandler();
            asyncHandler.addAsyncFilter(new CometAsyncFilter());
            st.setAsyncHandler(asyncHandler);
            
            st.setEnableAsyncExecution(true);
	        
	        
	        gws.start();
	
	        String url = null;
	        HttpURLConnection conn = null;
	        String response = null;
	        
	        /*
	        //JSP
	        url = context + servletPath + "/index.jsp";
	        conn = getConnection(url, PORT);
	        assertEquals(HttpServletResponse.SC_OK, getResponseCodeFromAlias(conn));
	
	        response = readResponse(conn);
	        assertEquals(url, response.trim());
	        */
	        //TWITTER
	        
	        url = "/grizzly-twitter/index.html";
	        conn = getConnection(url, PORT);
	        assertEquals(HttpServletResponse.SC_OK, getResponseCodeFromAlias(conn));
	
	        response = readResponse(conn);
	        assertEquals(url, response.trim());
	
	    } finally {
	        stopGrizzlyWebServer();
	    }
	}

	public void testJSP_JasperTwitter() throws IOException {
	    logger.info("testJSP_JasperTwitter");
	
	    try {
	        startGrizzlyWebServer(PORT);
	        
	        ServletAdapter adapter = new ServletAdapter();
	        Servlet servlet = new JspServlet();
	        adapter.setServletInstance(servlet);
	
	        adapter.setContextPath("/grizzly-twitter");
	        adapter.setServletPath("/");
	        adapter.setRootFolder("c:/temp/demo/grizzly-twitter");
	        
	        gws.addGrizzlyAdapter(adapter, new String[]{"/grizzly-twitter/*.jsp"});
	        
	        
	        // TWITTER 
	        ServletAdapter adapter2 = new ServletAdapter();
	        Servlet servlet2 = new TwitterServlet();
	        adapter2.setServletInstance(servlet2);
	        adapter2.addInitParameter("cometUrlPattern", "/griztter");
	        adapter2.addInitParameter("expirationDelay", "600000");
	
	        adapter2.setContextPath("/grizzly-twitter");
	        adapter2.setServletPath("/griztter");
	        adapter2.setRootFolder("c:/temp/demo/grizzly-twitter");
	
	        gws.addGrizzlyAdapter(adapter2, new String[]{"/grizzly-twitter/griztter"});
	        
	        
	        ServletAdapter adapter3 = new ServletAdapter();
			
	        adapter3.setContextPath("/grizzly-twitter");
			adapter3.setServletPath("/");
			adapter3.setHandleStaticResources(true);
			adapter3.setRootFolder("c:/temp/demo/grizzly-twitter");
			
			gws.addGrizzlyAdapter(adapter3, new String[]{"/grizzly-twitter/"});
	        
	        // COMET
	        SelectorThread st = gws.getSelectorThread();
	        
	        AsyncHandler asyncHandler = new DefaultAsyncHandler();
	        asyncHandler.addAsyncFilter(new CometAsyncFilter());
	        st.setAsyncHandler(asyncHandler);
	        
	        st.setEnableAsyncExecution(true);
	        
	        
	        gws.start();
	
	        String url = null;
	        HttpURLConnection conn = null;
	        String response = null;
	        
	        /*
	        //JSP
	        url = context + servletPath + "/index.jsp";
	        conn = getConnection(url, PORT);
	        assertEquals(HttpServletResponse.SC_OK, getResponseCodeFromAlias(conn));
	
	        response = readResponse(conn);
	        assertEquals(url, response.trim());
	        */
	        //TWITTER
	        
	        url = "/grizzly-twitter/index.html";
	        conn = getConnection(url, PORT);
	        assertEquals(HttpServletResponse.SC_OK, getResponseCodeFromAlias(conn));
	
	        response = readResponse(conn);
	        assertEquals(url, response.trim());
	
	    } finally {
	        stopGrizzlyWebServer();
	    }
	}
}
